//////////////////////////////////////////////////////////////////////////////
//           INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//      Copyright (c) 2000 Intel Corporation.  All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// IngressDispatch.uc - The dispatch loop for ingress side micro blocks.
//
// Description:
//    This file contains the dispatch loop for the ingress side micro blocks.
//    The specific example shown here is for the IXA SDK 2.0 ingress side processing.
//
//    This file is not meant to be a top-level source file.  This file should be
// 	  included in either the FastIngress or the SlowIngress source files.
// 

; Define some constants
#define SA_CONSUME_NUM     31
#define SEQNUM_IGNORE	   31
#define IX_EXCEPTION  		0

; Define variables that are global for the dispatch loop
//.local 	dl_reg1 dl_reg2 dl_reg3 dl_buffer_handle dl_next_block
.local dl_buffer_handle dl_next_block

#include "DispatchLoop_h.uc"
#include "DispatchLoopImportVars.h"
#include "EthernetIngress.uc"
#include "CountControlBlock.h"
#include "CountImportVars.h"
#include "bpf_filter.uc"

#ifdef SIMULATION
#include "Simulator_Init_h.uc"
SimulatorIngress_Init[]
#endif SIMULATION

; Do initialization
;initialize the variables for the dispatch loop
DL_Init[]
EthernetIngress_Init[]
filter_Init[]

; Run the dispatch loop
.while(1)
	;this macro is called at the beginning of the dispatch loop to poll the SA-to-ME ring
	DL_SASource[ ]
	.if (dl_buffer_handle == IX_BUFFER_NULL)
		br[Main_Dispatch#]
	; insert buffer at Filter
	.elif (dl_next_block == COUNT_TAG)
		br[Filter#]
	.else
		br[DL_Drop#]
	.endif

Main_Dispatch#:
	; ME (main) dispatch loop
	EthernetIngress[ ]
	.if (dl_buffer_handle == IX_BUFFER_NULL)
		.continue
	.endif

	.if (dl_next_block != 1)
		br[DL_Drop#]
	.else
Filter#:
		filter()
		.if (dl_next_block == IX_EXCEPTION)
			.local ace_tag
			immed32[ace_tag, COUNT_TAG]
			DL_SetAceTag[ace_tag]
			.endlocal
			br[DL_SASink#]
		.elif (dl_next_block == 1)
			br[DL_MESink#]
		.else
			br[DL_Drop#]
		.endif				
	.endif

DL_SASink#:
	DL_SASink[ ]	;this macro is called whenever a microblock returns the value IX_EXCEPTION
	.continue

DL_MESink#:
	DL_MESink[ ]	;this macro is called at the end of the dispatch loop to pass the packet onto the next microblock group
	ingress_eop#:			; for simulation perf measurements
	nop
	.continue

DL_Drop#:
	DL_Drop[ ]	;this macro is used by the dispatch loop to stop processing of a buffer

.endw
nop
nop
nop
.endlocal
