//typedef unsigned int u_int;

//typedef	int bpf_int32;
//typedef	u_int bpf_u_int32;

#define BPF_MAXINSNS 512
#define BPF_MAXBUFSIZE 0x8000
#define BPF_MINBUFSIZE 32

#define SHORT_SIZE 16
#define CHAR_SIZE   8

#ifndef MICROCODE
/*
 *  Structure for BIOCSETF.
 */
struct bpf_program {
	unsigned int bf_len;
	struct bpf_insn *bf_insns;
};
 
/*
 * Struct returned by BIOCGSTATS.
 */
struct bpf_stat {
	unsigned int bs_recv;		/* number of packets received */
	unsigned int bs_drop;		/* number of packets dropped */
};

/*
 * The instruction data structure.
 */
struct bpf_insn {
	struct {
	   	unsigned int code 			: 16;
	  	unsigned int jt 			:  8;
	  	unsigned int jf 			:  8;
	} instr;
	int k;
};
#endif
/*
 * The instruction encodings.
 */
/* instruction classes */
#define BPF_CLASS(code) ((code) & 0x07)
#define		BPF_LD		0x00
#define		BPF_LDX		0x01
#define		BPF_ST		0x02
#define		BPF_STX		0x03
#define		BPF_ALU		0x04
#define		BPF_JMP		0x05
#define		BPF_RET		0x06
#define		BPF_MISC	0x07

/* ld/ldx fields */
#define BPF_SIZE(code)	((code) & 0x18)
#define		BPF_W		0x00
#define		BPF_H		0x08
#define		BPF_B		0x10
#define BPF_MODE(code)	((code) & 0xe0)
#define		BPF_IMM 	0x00
#define		BPF_ABS		0x20
#define		BPF_IND		0x40
#define		BPF_MEM		0x60
#define		BPF_LEN		0x80
#define		BPF_MSH		0xa0

/* alu/jmp fields */
#define BPF_OP(code)	((code) & 0xf0)
#define		BPF_ADD		0x00
#define		BPF_SUB		0x10
#define		BPF_MUL		0x20
#define		BPF_DIV		0x30
#define		BPF_OR		0x40
#define		BPF_AND		0x50
#define		BPF_LSH		0x60
#define		BPF_RSH		0x70
#define		BPF_NEG		0x80
#define		BPF_JA		0x00
#define		BPF_JEQ		0x10
#define		BPF_JGT		0x20
#define		BPF_JGE		0x30
#define		BPF_JSET	0x40
#define BPF_SRC(code)	((code) & 0x08)
#define		BPF_K		0x00
#define		BPF_X		0x08

/* ret - BPF_K and BPF_X also apply */
#define BPF_RVAL(code)	((code) & 0x18)
#define		BPF_A		0x10

/* misc */
#define BPF_MISCOP(code) ((code) & 0xf8)
#define		BPF_TAX		0x00
#define		BPF_TXA		0x80



/*
 * Macros for insn array initializers.
 */
//#define BPF_STMT(code, k) { (u_short)(code), 0, 0, k }
//#define BPF_JUMP(code, k, jt, jf) { (u_short)(code), jt, jf, k }
#define BPF_STMT(code, k) { { code, 0, 0 }, k }
#define BPF_JUMP(code, k, jt, jf) { { code, jt, jf }, k }

/*
 * Number of scratch memory words (for BPF_LD|BPF_MEM and BPF_ST).
 */
#define BPF_MEMWORDS 16

#ifndef MICROCODE

#define EXTRACT_SHORT(p) *p
#define EXTRACT_LONG(p) *p

/* ExtractDataByte macro */
#define EXTRACT_DATA_BYTE(offset, p, A) { \
	switch (offset & 0x00000003) { \
	  case 0: \
	     A = p[offset >> 2] & 0xff000000; \
		 break; \
      case 1: \
	     A = p[offset >> 2] & 0x00ff0000 << 8; \
		 break; \
 	  case 2: \
	     A = p[offset >> 2] & 0x0000ff00 << 16; \
		 break; \
      case 3: \
	  	 A = p[offset >> 2] & 0x000000ff << 24; \
		 break; \
    } \
} 

unsigned int bpf_filter ( struct bpf_insn *pc,
	  					  unsigned int    *p,
						  unsigned int wirelen,
						  unsigned int buflen ) ;
#endif